﻿using System.Collections.Generic;
using System.Collections.Specialized;

namespace Nava.Web.UI
{
    public static class RequestExtensions
    {
        /// <summary>
        /// Retrieves the submitted action (string value "submit-action") value from the request. Typically, this is the name of the <button /> and with a required name of "submit-action".
        /// NOTE: if IE9 support is not needed, consider using the "formaction" attribute on the button element.
        /// </summary>
        /// <param name="collection"></param>
        /// <param name="action">Name of submitted action value</param>
        /// <returns></returns>
        public static bool HasSubmitAction(this NameValueCollection collection, string action)
        {
            var submittedAction = collection["submit-action"];

            return submittedAction != null && submittedAction.Equals(action, System.StringComparison.OrdinalIgnoreCase);
        }

        /// <summary>
        /// Retrieves form data in a key/value pair collection
        /// </summary>
        /// <param name="collection"></param>
        /// <returns></returns>
        public static IEnumerable<KeyValuePair<string, string>> GetParameters(this NameValueCollection collection)
        {
            foreach (string k in collection.Keys)
            {
                yield return new KeyValuePair<string, string>(k, collection[k]);
            }
        }
    }
}