﻿using BinaryDad.Extensions;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Web.Mvc;
using System.Web.Mvc.Html;
using System.Web.WebPages.Html;

namespace Nava.Web.UI
{
    public static class ViewExtensions
    {
        private const string DateFormat = "{0:M/d/yyyy}";

        #region HtmlHelper

        //public static object GetRouteValue(this HtmlHelper html, string key) => html.ViewContext.RouteData.Values[key];

        #region TextBoxFor<>

        public static MvcHtmlString TextBoxFor<TModel, TProperty>(this HtmlHelper<TModel> htmlHelper, Expression<Func<TModel, TProperty>> expression, object htmlAttributes, bool enabled) => htmlHelper.TextBoxFor(expression, htmlAttributes.GetPropertyValues(), enabled);

        public static MvcHtmlString TextBoxFor<TModel, TProperty>(this HtmlHelper<TModel> htmlHelper, Expression<Func<TModel, TProperty>> expression, IDictionary<string, object> htmlAttributes, bool enabled)
        {
            if (!enabled)
            {
                htmlAttributes.Add("disabled", "disabled");
            }

            return htmlHelper.TextBoxFor(expression, htmlAttributes);
        }

        #endregion

        #region TextAreaFor<>

        public static MvcHtmlString TextAreaFor<TModel, TProperty>(this HtmlHelper<TModel> htmlHelper, Expression<Func<TModel, TProperty>> expression, object htmlAttributes, bool enabled) => htmlHelper.TextAreaFor(expression, htmlAttributes.GetPropertyValues(), enabled);

        public static MvcHtmlString TextAreaFor<TModel, TProperty>(this HtmlHelper<TModel> htmlHelper, Expression<Func<TModel, TProperty>> expression, IDictionary<string, object> htmlAttributes, bool enabled)
        {
            if (!enabled)
            {
                htmlAttributes.Add("disabled", "disabled");
            }

            return htmlHelper.TextAreaFor(expression, htmlAttributes);
        }

        #endregion

        #region DropDownListFor<>

        public static MvcHtmlString DropDownListFor<TModel, TProperty>(this HtmlHelper<TModel> htmlHelper, Expression<Func<TModel, TProperty>> expression, IEnumerable<SelectListItem> selectList, string optionLabel, object htmlAttributes, bool enabled)
        {
            var properties = htmlAttributes.GetPropertyValues();

            if (!enabled)
            {
                properties.Add("disabled", "disabled");
            }

            return htmlHelper.DropDownListFor(expression, selectList, optionLabel, properties);
        }

        #endregion

        #region DropDownList

        public static MvcHtmlString DropDownList(this HtmlHelper htmlHelper, string name, IEnumerable<SelectListItem> selectList, object htmlAttributes, bool enabled) => htmlHelper.DropDownList(name, selectList, null, htmlAttributes, enabled);

        public static MvcHtmlString DropDownList(this HtmlHelper htmlHelper, string name, IEnumerable<SelectListItem> selectList, string optionLabel, object htmlAttributes, bool enabled)
        {
            var properties = htmlAttributes.GetPropertyValues();

            if (!enabled)
            {
                properties.Add("disabled", "disabled");
            }

            return htmlHelper.DropDownList(name, selectList, optionLabel, properties);
        }

        #endregion

        #region CheckBox

        public static MvcHtmlString CheckBox(this HtmlHelper htmlHelper, string name, bool isChecked, object htmlAttributes, bool enabled)
        {
            var properties = htmlAttributes.GetPropertyValues();

            if (!enabled)
            {
                properties.Add("disabled", "disabled");
            }

            return htmlHelper.CheckBox(name, isChecked, properties);
        }

        #endregion

        #region DateBoxFor<>

        public static MvcHtmlString DateBoxFor<TModel, TProperty>(this HtmlHelper<TModel> html, Expression<Func<TModel, TProperty>> expression) => html.DateBoxFor(expression, null);

        public static MvcHtmlString DateBoxFor<TModel, TProperty>(this HtmlHelper<TModel> html, Expression<Func<TModel, TProperty>> expression, bool enabled) => html.DateBoxFor(expression, null, enabled);

        public static MvcHtmlString DateBoxFor<TModel, TProperty>(this HtmlHelper<TModel> html, Expression<Func<TModel, TProperty>> expression, object htmlAttributes) => html.DateBoxFor(expression, htmlAttributes, true);

        public static MvcHtmlString DateBoxFor<TModel, TProperty>(this HtmlHelper<TModel> html, Expression<Func<TModel, TProperty>> expression, object htmlAttributes, bool enabled)
        {
            var properties = htmlAttributes.GetPropertyValues() ?? new Dictionary<string, object>();

            UpdateDateBoxProperties(properties, enabled);

            return html.TextBoxFor(expression, DateFormat, properties);
        }

        public static MvcHtmlString DateBox(this HtmlHelper html, string name, object value) => html.DateBox(name, value, null);

        public static MvcHtmlString DateBox(this HtmlHelper html, string name, object value, object htmlAttributes) => html.DateBox(name, value, htmlAttributes, true);

        public static MvcHtmlString DateBox(this HtmlHelper html, string name, object value, object htmlAttributes, bool enabled)
        {
            var properties = htmlAttributes.GetPropertyValues() ?? new Dictionary<string, object>();

            UpdateDateBoxProperties(properties, enabled);

            return html.TextBox(name, value, DateFormat, properties);
        }

        #endregion

        #region EnumDropDownListFor<>

        public static MvcHtmlString EnumDropDownListFor<TModel, TEnum>(this HtmlHelper<TModel> htmlHelper, Expression<Func<TModel, TEnum>> expression, string optionLabel, object htmlAttributes, bool useDescription)
        {
            // shouldnt happen here, but just in case
            if (!useDescription)
            {
                return htmlHelper.EnumDropDownListFor(expression, optionLabel, htmlAttributes);
            }

            var type = typeof(TEnum);

            if (type.IsNullable())
            {
                type = Nullable.GetUnderlyingType(type);
            }

            var values = Enum.GetValues(type)
                .Cast<TEnum>()
                .Select(v => new
                {
                    Value = Convert.ToInt32(v),
                    Text = (v as Enum).GetDescription()
                })
                .ToList();

            var list = new SelectList(values, "Value", "Text");

            return htmlHelper.DropDownListFor(expression, list, optionLabel, htmlAttributes);
        }

        #endregion

        #region ModalActionLink

        public static MvcHtmlString ModalActionLink(this HtmlHelper html, string linkText, string actionName, object routeValues, string dataTarget) => html.ModalActionLink(linkText, actionName, routeValues, dataTarget, null);

        public static MvcHtmlString ModalActionLink(this HtmlHelper html, string linkText, string actionName, object routeValues, string dataTarget, object htmlAttributes) => html.ModalActionLink(linkText, actionName, null, routeValues, dataTarget, htmlAttributes);

        public static MvcHtmlString ModalActionLink(this HtmlHelper html, string linkText, string actionName, string controllerName, object routeValues, string dataTarget) => html.ModalActionLink(linkText, actionName, controllerName, routeValues, dataTarget, null);

        public static MvcHtmlString ModalActionLink(this HtmlHelper html, string linkText, string actionName, string controllerName, object routeValues, string dataTarget, object htmlAttributes) => html.ModalActionLink(linkText, actionName, controllerName, routeValues, dataTarget, htmlAttributes, true);

        public static MvcHtmlString ModalActionLink(this HtmlHelper html, string linkText, string actionName, string controllerName, object routeValues, string dataTarget, object htmlAttributes, bool enabled)
        {
            var properties = htmlAttributes.GetPropertyValues() ?? new Dictionary<string, object>();

            MergeProperties(properties, GetModalProperties(dataTarget));

            if (!enabled)
            {
                properties.Add("disabled", "disabled");
            }

            return html.ActionLink(linkText, actionName, controllerName, new RouteValueDictionary(routeValues), properties);
        }

        #endregion

        #region ModalLink

        public static MvcHtmlString ModalLink(this HtmlHelper html, string linkText, string url, string dataTarget, object htmlAttributes)
        {
            var properties = htmlAttributes.GetPropertyValues() ?? new Dictionary<string, object>();
            var builder = new TagBuilder("a");

            builder.MergeAttributes(properties);
            builder.MergeAttributes(GetModalProperties(dataTarget).ToDictionary());
            builder.MergeAttribute("href", url);
            builder.SetInnerText(linkText);

            return MvcHtmlString.Create(builder.ToString());
        }

        #endregion

        #endregion

        #region UrlHelper

        public static string Action(this UrlHelper url, string actionName, string controllerName, object routeValues, string returnUrl, string returnTitle)
        {
            var values = new RouteValueDictionary(routeValues)
                .AddCreate("returnUrl", returnUrl)
                .AddCreate("returnTitle", returnTitle);

            return url.Action(actionName, controllerName, values);
        }

        #endregion

        #region ViewData

        public static T Get<T>(this ViewDataDictionary viewData, string key) where T : class => viewData[key] as T;

        #endregion

        #region Private methods

        private static IEnumerable<KeyValuePair<string, object>> GetModalProperties(string dataTarget)
        {
            return new Dictionary<string, object>
            {
                { "data-toggle", "modal" },
                { "data-target", dataTarget }
            };
        }

        private static void MergeProperties(IDictionary<string, object> source, IEnumerable<KeyValuePair<string, object>> merge)
        {
            if (source != null && merge != null)
            {
                merge.ForEach(m =>
                {
                    if (source.ContainsKey(m.Key))
                    {
                        source[m.Key] += String.Format(" {0}", m.Value.ToString());
                    }
                    else
                    {
                        source.Add(m.Key, m.Value);
                    }
                });
            }
        }

        private static void UpdateDateBoxProperties(IDictionary<string, object> properties, bool enabled)
        {
            MergeProperties(properties, new Dictionary<string, object>
            {
                { "class", "datepicker form-control enrollmentdate edit-not-allowed" },
                { "data-provide", "datepicker" },
                { "data-date-format", "m/d/yyyy" },
                { "readonly", "readonly" } // this also hides the keyboard in iOS
            });

            if (!enabled)
            {
                properties.Add("disabled", "disabled");
            }
        }

        #endregion
    }
}